/* these styles are in the head of this page because this is a unique page */

/* THE BIG GUYS */
html {
	direction: rtl;
}

/* LISTS */
ul {
	margin: .3em 2em 1.5em 0;
}

li {
	margin-right: 2em;
}

#wrap {
	text-align: right;
}

/* TYPOGRAPHY */
dl dd {
	margin-right: 3em;
}

.efieldlist {
	border-right: 1px solid #e6e6e6;
}

div.caution {
	padding: .2em 60px .2em .2em;
	background-position: right;
}

th.rowtitle {
        text-align: right;
}
.enablebox table {
	margin-right: 1em;
}
.enablebox.mini table {
	float: left;
}
.enablebox tr td {
	padding: .5px .5em 1px 1em;
}

/* Attribute presentation in example page */
table.attributes td.attrname {
	text-align: left;
}

fieldset.fancyfieldset {
	margin: 2em 0px 1em 1em;
}
fieldset.fancyfieldset legend {
	margin-right: 2em;
}

.ui-tabs .ui-tabs-nav li { 
	float: right; 
}

/* Reverse Float Left <-> Right */
.float-r {
	float: left;
}
.float-l {
	float: right;
}
