<?php

// autoload_classmap.php @generated by Composer

$vendorDir = dirname(dirname(__FILE__));
$baseDir = dirname($vendorDir);

return array(
    'Auth_OpenID' => $vendorDir . '/openid/php-openid/Auth/OpenID.php',
    'Auth_OpenID_AX' => $vendorDir . '/openid/php-openid/Auth/OpenID/AX.php',
    'Auth_OpenID_AX_AttrInfo' => $vendorDir . '/openid/php-openid/Auth/OpenID/AX.php',
    'Auth_OpenID_AX_Error' => $vendorDir . '/openid/php-openid/Auth/OpenID/AX.php',
    'Auth_OpenID_AX_FetchRequest' => $vendorDir . '/openid/php-openid/Auth/OpenID/AX.php',
    'Auth_OpenID_AX_FetchResponse' => $vendorDir . '/openid/php-openid/Auth/OpenID/AX.php',
    'Auth_OpenID_AX_KeyValueMessage' => $vendorDir . '/openid/php-openid/Auth/OpenID/AX.php',
    'Auth_OpenID_AX_Message' => $vendorDir . '/openid/php-openid/Auth/OpenID/AX.php',
    'Auth_OpenID_AX_StoreRequest' => $vendorDir . '/openid/php-openid/Auth/OpenID/AX.php',
    'Auth_OpenID_AX_StoreResponse' => $vendorDir . '/openid/php-openid/Auth/OpenID/AX.php',
    'Auth_OpenID_AlreadySigned' => $vendorDir . '/openid/php-openid/Auth/OpenID/Server.php',
    'Auth_OpenID_AssociateRequest' => $vendorDir . '/openid/php-openid/Auth/OpenID/Server.php',
    'Auth_OpenID_Association' => $vendorDir . '/openid/php-openid/Auth/OpenID/Association.php',
    'Auth_OpenID_AuthRequest' => $vendorDir . '/openid/php-openid/Auth/OpenID/Consumer.php',
    'Auth_OpenID_BcMathWrapper' => $vendorDir . '/openid/php-openid/Auth/OpenID/BigMath.php',
    'Auth_OpenID_CancelResponse' => $vendorDir . '/openid/php-openid/Auth/OpenID/Consumer.php',
    'Auth_OpenID_CheckAuthRequest' => $vendorDir . '/openid/php-openid/Auth/OpenID/Server.php',
    'Auth_OpenID_CheckIDRequest' => $vendorDir . '/openid/php-openid/Auth/OpenID/Server.php',
    'Auth_OpenID_Consumer' => $vendorDir . '/openid/php-openid/Auth/OpenID/Consumer.php',
    'Auth_OpenID_ConsumerResponse' => $vendorDir . '/openid/php-openid/Auth/OpenID/Consumer.php',
    'Auth_OpenID_CryptUtil' => $vendorDir . '/openid/php-openid/Auth/OpenID/CryptUtil.php',
    'Auth_OpenID_DatabaseConnection' => $vendorDir . '/openid/php-openid/Auth/OpenID/DatabaseConnection.php',
    'Auth_OpenID_Decoder' => $vendorDir . '/openid/php-openid/Auth/OpenID/Server.php',
    'Auth_OpenID_DiffieHellman' => $vendorDir . '/openid/php-openid/Auth/OpenID/DiffieHellman.php',
    'Auth_OpenID_DiffieHellmanSHA1ConsumerSession' => $vendorDir . '/openid/php-openid/Auth/OpenID/Consumer.php',
    'Auth_OpenID_DiffieHellmanSHA1ServerSession' => $vendorDir . '/openid/php-openid/Auth/OpenID/Server.php',
    'Auth_OpenID_DiffieHellmanSHA256ConsumerSession' => $vendorDir . '/openid/php-openid/Auth/OpenID/Consumer.php',
    'Auth_OpenID_DiffieHellmanSHA256ServerSession' => $vendorDir . '/openid/php-openid/Auth/OpenID/Server.php',
    'Auth_OpenID_DumbStore' => $vendorDir . '/openid/php-openid/Auth/OpenID/DumbStore.php',
    'Auth_OpenID_Encoder' => $vendorDir . '/openid/php-openid/Auth/OpenID/Server.php',
    'Auth_OpenID_EncodingError' => $vendorDir . '/openid/php-openid/Auth/OpenID/Server.php',
    'Auth_OpenID_Extension' => $vendorDir . '/openid/php-openid/Auth/OpenID/Extension.php',
    'Auth_OpenID_FailureResponse' => $vendorDir . '/openid/php-openid/Auth/OpenID/Consumer.php',
    'Auth_OpenID_FileStore' => $vendorDir . '/openid/php-openid/Auth/OpenID/FileStore.php',
    'Auth_OpenID_GenericConsumer' => $vendorDir . '/openid/php-openid/Auth/OpenID/Consumer.php',
    'Auth_OpenID_GmpMathWrapper' => $vendorDir . '/openid/php-openid/Auth/OpenID/BigMath.php',
    'Auth_OpenID_KVForm' => $vendorDir . '/openid/php-openid/Auth/OpenID/KVForm.php',
    'Auth_OpenID_MDB2Store' => $vendorDir . '/openid/php-openid/Auth/OpenID/MDB2Store.php',
    'Auth_OpenID_MalformedReturnURL' => $vendorDir . '/openid/php-openid/Auth/OpenID/Server.php',
    'Auth_OpenID_MalformedTrustRoot' => $vendorDir . '/openid/php-openid/Auth/OpenID/Server.php',
    'Auth_OpenID_Mapping' => $vendorDir . '/openid/php-openid/Auth/OpenID/Message.php',
    'Auth_OpenID_MathLibrary' => $vendorDir . '/openid/php-openid/Auth/OpenID/BigMath.php',
    'Auth_OpenID_MemcachedStore' => $vendorDir . '/openid/php-openid/Auth/OpenID/MemcachedStore.php',
    'Auth_OpenID_Message' => $vendorDir . '/openid/php-openid/Auth/OpenID/Message.php',
    'Auth_OpenID_MySQLStore' => $vendorDir . '/openid/php-openid/Auth/OpenID/MySQLStore.php',
    'Auth_OpenID_NamespaceMap' => $vendorDir . '/openid/php-openid/Auth/OpenID/Message.php',
    'Auth_OpenID_NoReturnToError' => $vendorDir . '/openid/php-openid/Auth/OpenID/Server.php',
    'Auth_OpenID_OpenIDStore' => $vendorDir . '/openid/php-openid/Auth/OpenID/Interface.php',
    'Auth_OpenID_PAPE_Request' => $vendorDir . '/openid/php-openid/Auth/OpenID/PAPE.php',
    'Auth_OpenID_PAPE_Response' => $vendorDir . '/openid/php-openid/Auth/OpenID/PAPE.php',
    'Auth_OpenID_Parse' => $vendorDir . '/openid/php-openid/Auth/OpenID/Parse.php',
    'Auth_OpenID_PlainTextConsumerSession' => $vendorDir . '/openid/php-openid/Auth/OpenID/Consumer.php',
    'Auth_OpenID_PlainTextServerSession' => $vendorDir . '/openid/php-openid/Auth/OpenID/Server.php',
    'Auth_OpenID_PostgreSQLStore' => $vendorDir . '/openid/php-openid/Auth/OpenID/PostgreSQLStore.php',
    'Auth_OpenID_PredisStore' => $vendorDir . '/openid/php-openid/Auth/OpenID/PredisStore.php',
    'Auth_OpenID_Request' => $vendorDir . '/openid/php-openid/Auth/OpenID/Server.php',
    'Auth_OpenID_SQLStore' => $vendorDir . '/openid/php-openid/Auth/OpenID/SQLStore.php',
    'Auth_OpenID_SQLiteStore' => $vendorDir . '/openid/php-openid/Auth/OpenID/SQLiteStore.php',
    'Auth_OpenID_SRegBase' => $vendorDir . '/openid/php-openid/Auth/OpenID/SReg.php',
    'Auth_OpenID_SRegRequest' => $vendorDir . '/openid/php-openid/Auth/OpenID/SReg.php',
    'Auth_OpenID_SRegResponse' => $vendorDir . '/openid/php-openid/Auth/OpenID/SReg.php',
    'Auth_OpenID_Server' => $vendorDir . '/openid/php-openid/Auth/OpenID/Server.php',
    'Auth_OpenID_ServerError' => $vendorDir . '/openid/php-openid/Auth/OpenID/Server.php',
    'Auth_OpenID_ServerErrorContainer' => $vendorDir . '/openid/php-openid/Auth/OpenID/Consumer.php',
    'Auth_OpenID_ServerRequest' => $vendorDir . '/openid/php-openid/Auth/OpenID/ServerRequest.php',
    'Auth_OpenID_ServerResponse' => $vendorDir . '/openid/php-openid/Auth/OpenID/Server.php',
    'Auth_OpenID_ServiceEndpoint' => $vendorDir . '/openid/php-openid/Auth/OpenID/Discover.php',
    'Auth_OpenID_ServiceEndpointLoader' => $vendorDir . '/openid/php-openid/Auth/Yadis/Manager.php',
    'Auth_OpenID_SessionNegotiator' => $vendorDir . '/openid/php-openid/Auth/OpenID/Association.php',
    'Auth_OpenID_SetupNeededResponse' => $vendorDir . '/openid/php-openid/Auth/OpenID/Consumer.php',
    'Auth_OpenID_Signatory' => $vendorDir . '/openid/php-openid/Auth/OpenID/Server.php',
    'Auth_OpenID_SigningEncoder' => $vendorDir . '/openid/php-openid/Auth/OpenID/Server.php',
    'Auth_OpenID_SuccessResponse' => $vendorDir . '/openid/php-openid/Auth/OpenID/Consumer.php',
    'Auth_OpenID_TrustRoot' => $vendorDir . '/openid/php-openid/Auth/OpenID/TrustRoot.php',
    'Auth_OpenID_TypeURIMismatch' => $vendorDir . '/openid/php-openid/Auth/OpenID/Consumer.php',
    'Auth_OpenID_UntrustedReturnURL' => $vendorDir . '/openid/php-openid/Auth/OpenID/Server.php',
    'Auth_OpenID_WebResponse' => $vendorDir . '/openid/php-openid/Auth/OpenID/Server.php',
    'Auth_Yadis_Discovery' => $vendorDir . '/openid/php-openid/Auth/Yadis/Manager.php',
    'Auth_Yadis_DiscoveryResult' => $vendorDir . '/openid/php-openid/Auth/Yadis/Yadis.php',
    'Auth_Yadis_HTTPFetcher' => $vendorDir . '/openid/php-openid/Auth/Yadis/HTTPFetcher.php',
    'Auth_Yadis_HTTPResponse' => $vendorDir . '/openid/php-openid/Auth/Yadis/HTTPFetcher.php',
    'Auth_Yadis_Manager' => $vendorDir . '/openid/php-openid/Auth/Yadis/Manager.php',
    'Auth_Yadis_ManagerLoader' => $vendorDir . '/openid/php-openid/Auth/Yadis/Manager.php',
    'Auth_Yadis_PHPSession' => $vendorDir . '/openid/php-openid/Auth/Yadis/Manager.php',
    'Auth_Yadis_ParanoidHTTPFetcher' => $vendorDir . '/openid/php-openid/Auth/Yadis/ParanoidHTTPFetcher.php',
    'Auth_Yadis_ParseHTML' => $vendorDir . '/openid/php-openid/Auth/Yadis/ParseHTML.php',
    'Auth_Yadis_PlainHTTPFetcher' => $vendorDir . '/openid/php-openid/Auth/Yadis/PlainHTTPFetcher.php',
    'Auth_Yadis_ProxyResolver' => $vendorDir . '/openid/php-openid/Auth/Yadis/XRIRes.php',
    'Auth_Yadis_Service' => $vendorDir . '/openid/php-openid/Auth/Yadis/XRDS.php',
    'Auth_Yadis_SessionLoader' => $vendorDir . '/openid/php-openid/Auth/Yadis/Manager.php',
    'Auth_Yadis_XMLParser' => $vendorDir . '/openid/php-openid/Auth/Yadis/XML.php',
    'Auth_Yadis_XRDS' => $vendorDir . '/openid/php-openid/Auth/Yadis/XRDS.php',
    'Auth_Yadis_Yadis' => $vendorDir . '/openid/php-openid/Auth/Yadis/Yadis.php',
    'Auth_Yadis_dom' => $vendorDir . '/openid/php-openid/Auth/Yadis/XML.php',
    'Auth_Yadis_domxml' => $vendorDir . '/openid/php-openid/Auth/Yadis/XML.php',
    'Psr\\Log\\AbstractLogger' => $vendorDir . '/psr/log/Psr/Log/AbstractLogger.php',
    'Psr\\Log\\InvalidArgumentException' => $vendorDir . '/psr/log/Psr/Log/InvalidArgumentException.php',
    'Psr\\Log\\LogLevel' => $vendorDir . '/psr/log/Psr/Log/LogLevel.php',
    'Psr\\Log\\LoggerAwareInterface' => $vendorDir . '/psr/log/Psr/Log/LoggerAwareInterface.php',
    'Psr\\Log\\LoggerAwareTrait' => $vendorDir . '/psr/log/Psr/Log/LoggerAwareTrait.php',
    'Psr\\Log\\LoggerInterface' => $vendorDir . '/psr/log/Psr/Log/LoggerInterface.php',
    'Psr\\Log\\LoggerTrait' => $vendorDir . '/psr/log/Psr/Log/LoggerTrait.php',
    'Psr\\Log\\NullLogger' => $vendorDir . '/psr/log/Psr/Log/NullLogger.php',
    'Psr\\Log\\Test\\DummyTest' => $vendorDir . '/psr/log/Psr/Log/Test/LoggerInterfaceTest.php',
    'Psr\\Log\\Test\\LoggerInterfaceTest' => $vendorDir . '/psr/log/Psr/Log/Test/LoggerInterfaceTest.php',
    'SAML2_ArtifactResolve' => $vendorDir . '/simplesamlphp/saml2/src/SAML2/ArtifactResolve.php',
    'SAML2_ArtifactResponse' => $vendorDir . '/simplesamlphp/saml2/src/SAML2/ArtifactResponse.php',
    'SAML2_Assertion' => $vendorDir . '/simplesamlphp/saml2/src/SAML2/Assertion.php',
    'SAML2_Assertion_Decrypter' => $vendorDir . '/simplesamlphp/saml2/src/SAML2/Assertion/Decrypter.php',
    'SAML2_Assertion_Exception_InvalidAssertionException' => $vendorDir . '/simplesamlphp/saml2/src/SAML2/Assertion/Exception/InvalidAssertionException.php',
    'SAML2_Assertion_Exception_InvalidSubjectConfirmationException' => $vendorDir . '/simplesamlphp/saml2/src/SAML2/Assertion/Exception/InvalidSubjectConfirmationException.php',
    'SAML2_Assertion_Exception_NotDecryptedException' => $vendorDir . '/simplesamlphp/saml2/src/SAML2/Assertion/Exception/NotDecryptedException.php',
    'SAML2_Assertion_Processor' => $vendorDir . '/simplesamlphp/saml2/src/SAML2/Assertion/Processor.php',
    'SAML2_Assertion_ProcessorBuilder' => $vendorDir . '/simplesamlphp/saml2/src/SAML2/Assertion/ProcessorBuilder.php',
    'SAML2_Assertion_Transformer_DecodeBase64Transformer' => $vendorDir . '/simplesamlphp/saml2/src/SAML2/Assertion/Transformer/DecodeBase64Transformer.php',
    'SAML2_Assertion_Transformer_NameIdDecryptionTransformer' => $vendorDir . '/simplesamlphp/saml2/src/SAML2/Assertion/Transformer/NameIdDecryptionTransformer.php',
    'SAML2_Assertion_Transformer_Transformer' => $vendorDir . '/simplesamlphp/saml2/src/SAML2/Assertion/Transformer/Transformer.php',
    'SAML2_Assertion_Transformer_TransformerChain' => $vendorDir . '/simplesamlphp/saml2/src/SAML2/Assertion/Transformer/TransformerChain.php',
    'SAML2_Assertion_Validation_AssertionConstraintValidator' => $vendorDir . '/simplesamlphp/saml2/src/SAML2/Assertion/Validation/AssertionConstraintValidator.php',
    'SAML2_Assertion_Validation_AssertionValidator' => $vendorDir . '/simplesamlphp/saml2/src/SAML2/Assertion/Validation/AssertionValidator.php',
    'SAML2_Assertion_Validation_ConstraintValidator_NotBefore' => $vendorDir . '/simplesamlphp/saml2/src/SAML2/Assertion/Validation/ConstraintValidator/NotBefore.php',
    'SAML2_Assertion_Validation_ConstraintValidator_NotOnOrAfter' => $vendorDir . '/simplesamlphp/saml2/src/SAML2/Assertion/Validation/ConstraintValidator/NotOnOrAfter.php',
    'SAML2_Assertion_Validation_ConstraintValidator_SessionNotOnOrAfter' => $vendorDir . '/simplesamlphp/saml2/src/SAML2/Assertion/Validation/ConstraintValidator/SessionNotOnOrAfter.php',
    'SAML2_Assertion_Validation_ConstraintValidator_SpIsValidAudience' => $vendorDir . '/simplesamlphp/saml2/src/SAML2/Assertion/Validation/ConstraintValidator/SpIsValidAudience.php',
    'SAML2_Assertion_Validation_ConstraintValidator_SubjectConfirmationMethod' => $vendorDir . '/simplesamlphp/saml2/src/SAML2/Assertion/Validation/ConstraintValidator/SubjectConfirmationMethod.php',
    'SAML2_Assertion_Validation_ConstraintValidator_SubjectConfirmationNotBefore' => $vendorDir . '/simplesamlphp/saml2/src/SAML2/Assertion/Validation/ConstraintValidator/SubjectConfirmationNotBefore.php',
    'SAML2_Assertion_Validation_ConstraintValidator_SubjectConfirmationNotOnOrAfter' => $vendorDir . '/simplesamlphp/saml2/src/SAML2/Assertion/Validation/ConstraintValidator/SubjectConfirmationNotOnOrAfter.php',
    'SAML2_Assertion_Validation_ConstraintValidator_SubjectConfirmationRecipientMatches' => $vendorDir . '/simplesamlphp/saml2/src/SAML2/Assertion/Validation/ConstraintValidator/SubjectConfirmationRecipientMatches.php',
    'SAML2_Assertion_Validation_ConstraintValidator_SubjectConfirmationResponseToMatches' => $vendorDir . '/simplesamlphp/saml2/src/SAML2/Assertion/Validation/ConstraintValidator/SubjectConfirmationResponseToMatches.php',
    'SAML2_Assertion_Validation_Result' => $vendorDir . '/simplesamlphp/saml2/src/SAML2/Assertion/Validation/Result.php',
    'SAML2_Assertion_Validation_SubjectConfirmationConstraintValidator' => $vendorDir . '/simplesamlphp/saml2/src/SAML2/Assertion/Validation/SubjectConfirmationConstraintValidator.php',
    'SAML2_Assertion_Validation_SubjectConfirmationValidator' => $vendorDir . '/simplesamlphp/saml2/src/SAML2/Assertion/Validation/SubjectConfirmationValidator.php',
    'SAML2_AttributeQuery' => $vendorDir . '/simplesamlphp/saml2/src/SAML2/AttributeQuery.php',
    'SAML2_AuthnRequest' => $vendorDir . '/simplesamlphp/saml2/src/SAML2/AuthnRequest.php',
    'SAML2_Binding' => $vendorDir . '/simplesamlphp/saml2/src/SAML2/Binding.php',
    'SAML2_Certificate_Exception_InvalidCertificateStructureException' => $vendorDir . '/simplesamlphp/saml2/src/SAML2/Certificate/Exception/InvalidCertificateStructureException.php',
    'SAML2_Certificate_Exception_InvalidKeyUsageException' => $vendorDir . '/simplesamlphp/saml2/src/SAML2/Certificate/Exception/InvalidKeyUsageException.php',
    'SAML2_Certificate_Exception_NoKeysFoundException' => $vendorDir . '/simplesamlphp/saml2/src/SAML2/Certificate/Exception/NoKeysFoundException.php',
    'SAML2_Certificate_Fingerprint' => $vendorDir . '/simplesamlphp/saml2/src/SAML2/Certificate/Fingerprint.php',
    'SAML2_Certificate_FingerprintCollection' => $vendorDir . '/simplesamlphp/saml2/src/SAML2/Certificate/FingerprintCollection.php',
    'SAML2_Certificate_FingerprintLoader' => $vendorDir . '/simplesamlphp/saml2/src/SAML2/Certificate/FingerprintLoader.php',
    'SAML2_Certificate_Key' => $vendorDir . '/simplesamlphp/saml2/src/SAML2/Certificate/Key.php',
    'SAML2_Certificate_KeyCollection' => $vendorDir . '/simplesamlphp/saml2/src/SAML2/Certificate/KeyCollection.php',
    'SAML2_Certificate_KeyLoader' => $vendorDir . '/simplesamlphp/saml2/src/SAML2/Certificate/KeyLoader.php',
    'SAML2_Certificate_PrivateKey' => $vendorDir . '/simplesamlphp/saml2/src/SAML2/Certificate/PrivateKey.php',
    'SAML2_Certificate_PrivateKeyLoader' => $vendorDir . '/simplesamlphp/saml2/src/SAML2/Certificate/PrivateKeyLoader.php',
    'SAML2_Certificate_X509' => $vendorDir . '/simplesamlphp/saml2/src/SAML2/Certificate/X509.php',
    'SAML2_Compat_AbstractContainer' => $vendorDir . '/simplesamlphp/saml2/src/SAML2/Compat/AbstractContainer.php',
    'SAML2_Compat_ContainerSingleton' => $vendorDir . '/simplesamlphp/saml2/src/SAML2/Compat/ContainerSingleton.php',
    'SAML2_Compat_MockContainer' => $vendorDir . '/simplesamlphp/saml2/src/SAML2/Compat/MockContainer.php',
    'SAML2_Compat_Ssp_Container' => $vendorDir . '/simplesamlphp/saml2/src/SAML2/Compat/Ssp/Container.php',
    'SAML2_Compat_Ssp_Logger' => $vendorDir . '/simplesamlphp/saml2/src/SAML2/Compat/Ssp/Logger.php',
    'SAML2_Configuration_ArrayAdapter' => $vendorDir . '/simplesamlphp/saml2/src/SAML2/Configuration/ArrayAdapter.php',
    'SAML2_Configuration_CertificateProvider' => $vendorDir . '/simplesamlphp/saml2/src/SAML2/Configuration/CertificateProvider.php',
    'SAML2_Configuration_DecryptionProvider' => $vendorDir . '/simplesamlphp/saml2/src/SAML2/Configuration/DecryptionProvider.php',
    'SAML2_Configuration_Destination' => $vendorDir . '/simplesamlphp/saml2/src/SAML2/Configuration/Destination.php',
    'SAML2_Configuration_EntityIdProvider' => $vendorDir . '/simplesamlphp/saml2/src/SAML2/Configuration/EntityIdProvider.php',
    'SAML2_Configuration_IdentityProvider' => $vendorDir . '/simplesamlphp/saml2/src/SAML2/Configuration/IdentityProvider.php',
    'SAML2_Configuration_IdentityProviderAware' => $vendorDir . '/simplesamlphp/saml2/src/SAML2/Configuration/IdentityProviderAware.php',
    'SAML2_Configuration_PrivateKey' => $vendorDir . '/simplesamlphp/saml2/src/SAML2/Configuration/PrivateKey.php',
    'SAML2_Configuration_Queryable' => $vendorDir . '/simplesamlphp/saml2/src/SAML2/Configuration/Queryable.php',
    'SAML2_Configuration_ServiceProvider' => $vendorDir . '/simplesamlphp/saml2/src/SAML2/Configuration/ServiceProvider.php',
    'SAML2_Configuration_ServiceProviderAware' => $vendorDir . '/simplesamlphp/saml2/src/SAML2/Configuration/ServiceProviderAware.php',
    'SAML2_Configuration_SimpleSAMLConverter' => $vendorDir . '/simplesamlphp/saml2/src/SAML2/Configuration/SimpleSAMLConverter.php',
    'SAML2_Const' => $vendorDir . '/simplesamlphp/saml2/src/SAML2/Const.php',
    'SAML2_DOMDocumentFactory' => $vendorDir . '/simplesamlphp/saml2/src/SAML2/DOMDocumentFactory.php',
    'SAML2_EncryptedAssertion' => $vendorDir . '/simplesamlphp/saml2/src/SAML2/EncryptedAssertion.php',
    'SAML2_Exception_InvalidArgumentException' => $vendorDir . '/simplesamlphp/saml2/src/SAML2/Exception/InvalidArgumentException.php',
    'SAML2_Exception_RuntimeException' => $vendorDir . '/simplesamlphp/saml2/src/SAML2/Exception/RuntimeException.php',
    'SAML2_Exception_Throwable' => $vendorDir . '/simplesamlphp/saml2/src/SAML2/Exception/Throwable.php',
    'SAML2_Exception_UnparseableXmlException' => $vendorDir . '/simplesamlphp/saml2/src/SAML2/Exception/UnparseableXmlException.php',
    'SAML2_HTTPArtifact' => $vendorDir . '/simplesamlphp/saml2/src/SAML2/HTTPArtifact.php',
    'SAML2_HTTPPost' => $vendorDir . '/simplesamlphp/saml2/src/SAML2/HTTPPost.php',
    'SAML2_HTTPRedirect' => $vendorDir . '/simplesamlphp/saml2/src/SAML2/HTTPRedirect.php',
    'SAML2_LogoutRequest' => $vendorDir . '/simplesamlphp/saml2/src/SAML2/LogoutRequest.php',
    'SAML2_LogoutResponse' => $vendorDir . '/simplesamlphp/saml2/src/SAML2/LogoutResponse.php',
    'SAML2_Message' => $vendorDir . '/simplesamlphp/saml2/src/SAML2/Message.php',
    'SAML2_Request' => $vendorDir . '/simplesamlphp/saml2/src/SAML2/Request.php',
    'SAML2_Response' => $vendorDir . '/simplesamlphp/saml2/src/SAML2/Response.php',
    'SAML2_Response_Exception_InvalidResponseException' => $vendorDir . '/simplesamlphp/saml2/src/SAML2/Response/Exception/InvalidResponseException.php',
    'SAML2_Response_Exception_InvalidSignatureException' => $vendorDir . '/simplesamlphp/saml2/src/SAML2/Response/Exception/InvalidSignatureException.php',
    'SAML2_Response_Exception_NoAssertionsFoundException' => $vendorDir . '/simplesamlphp/saml2/src/SAML2/Response/Exception/NoAssertionsFoundException.php',
    'SAML2_Response_Exception_PreconditionNotMetException' => $vendorDir . '/simplesamlphp/saml2/src/SAML2/Response/Exception/PreconditionNotMetException.php',
    'SAML2_Response_Exception_UnencryptedAssertionFoundException' => $vendorDir . '/simplesamlphp/saml2/src/SAML2/Response/Exception/UnencryptedAssertionFoundException.php',
    'SAML2_Response_Exception_UnsignedResponseException' => $vendorDir . '/simplesamlphp/saml2/src/SAML2/Response/Exception/UnsignedResponseException.php',
    'SAML2_Response_Processor' => $vendorDir . '/simplesamlphp/saml2/src/SAML2/Response/Processor.php',
    'SAML2_Response_Validation_ConstraintValidator' => $vendorDir . '/simplesamlphp/saml2/src/SAML2/Response/Validation/ConstraintValidator.php',
    'SAML2_Response_Validation_ConstraintValidator_DestinationMatches' => $vendorDir . '/simplesamlphp/saml2/src/SAML2/Response/Validation/ConstraintValidator/DestinationMatches.php',
    'SAML2_Response_Validation_ConstraintValidator_IsSuccessful' => $vendorDir . '/simplesamlphp/saml2/src/SAML2/Response/Validation/ConstraintValidator/IsSuccessful.php',
    'SAML2_Response_Validation_PreconditionValidator' => $vendorDir . '/simplesamlphp/saml2/src/SAML2/Response/Validation/PreconditionValidator.php',
    'SAML2_Response_Validation_Result' => $vendorDir . '/simplesamlphp/saml2/src/SAML2/Response/Validation/Result.php',
    'SAML2_Response_Validation_Validator' => $vendorDir . '/simplesamlphp/saml2/src/SAML2/Response/Validation/Validator.php',
    'SAML2_SOAP' => $vendorDir . '/simplesamlphp/saml2/src/SAML2/SOAP.php',
    'SAML2_SOAPClient' => $vendorDir . '/simplesamlphp/saml2/src/SAML2/SOAPClient.php',
    'SAML2_Signature_AbstractChainedValidator' => $vendorDir . '/simplesamlphp/saml2/src/SAML2/Signature/AbstractChainedValidator.php',
    'SAML2_Signature_ChainedValidator' => $vendorDir . '/simplesamlphp/saml2/src/SAML2/Signature/ChainedValidator.php',
    'SAML2_Signature_FingerprintValidator' => $vendorDir . '/simplesamlphp/saml2/src/SAML2/Signature/FingerprintValidator.php',
    'SAML2_Signature_MissingConfigurationException' => $vendorDir . '/simplesamlphp/saml2/src/SAML2/Signature/MissingConfigurationException.php',
    'SAML2_Signature_PublicKeyValidator' => $vendorDir . '/simplesamlphp/saml2/src/SAML2/Signature/PublicKeyValidator.php',
    'SAML2_Signature_Validator' => $vendorDir . '/simplesamlphp/saml2/src/SAML2/Signature/Validator.php',
    'SAML2_Signature_ValidatorChain' => $vendorDir . '/simplesamlphp/saml2/src/SAML2/Signature/ValidatorChain.php',
    'SAML2_Signature_ValidatorInterface' => $vendorDir . '/simplesamlphp/saml2/src/SAML2/Signature/ValidatorInterface.php',
    'SAML2_SignedElement' => $vendorDir . '/simplesamlphp/saml2/src/SAML2/SignedElement.php',
    'SAML2_SignedElementHelper' => $vendorDir . '/simplesamlphp/saml2/src/SAML2/SignedElementHelper.php',
    'SAML2_StatusResponse' => $vendorDir . '/simplesamlphp/saml2/src/SAML2/StatusResponse.php',
    'SAML2_SubjectQuery' => $vendorDir . '/simplesamlphp/saml2/src/SAML2/SubjectQuery.php',
    'SAML2_Utilities_ArrayCollection' => $vendorDir . '/simplesamlphp/saml2/src/SAML2/Utilities/ArrayCollection.php',
    'SAML2_Utilities_Certificate' => $vendorDir . '/simplesamlphp/saml2/src/SAML2/Utilities/Certificate.php',
    'SAML2_Utilities_Collection' => $vendorDir . '/simplesamlphp/saml2/src/SAML2/Utilities/Collection.php',
    'SAML2_Utilities_File' => $vendorDir . '/simplesamlphp/saml2/src/SAML2/Utilities/File.php',
    'SAML2_Utilities_Temporal' => $vendorDir . '/simplesamlphp/saml2/src/SAML2/Utilities/Temporal.php',
    'SAML2_Utils' => $vendorDir . '/simplesamlphp/saml2/src/SAML2/Utils.php',
    'SAML2_XML_Chunk' => $vendorDir . '/simplesamlphp/saml2/src/SAML2/XML/Chunk.php',
    'SAML2_XML_ds_KeyInfo' => $vendorDir . '/simplesamlphp/saml2/src/SAML2/XML/ds/KeyInfo.php',
    'SAML2_XML_ds_KeyName' => $vendorDir . '/simplesamlphp/saml2/src/SAML2/XML/ds/KeyName.php',
    'SAML2_XML_ds_X509Certificate' => $vendorDir . '/simplesamlphp/saml2/src/SAML2/XML/ds/X509Certificate.php',
    'SAML2_XML_ds_X509Data' => $vendorDir . '/simplesamlphp/saml2/src/SAML2/XML/ds/X509Data.php',
    'SAML2_XML_md_AdditionalMetadataLocation' => $vendorDir . '/simplesamlphp/saml2/src/SAML2/XML/md/AdditionalMetadataLocation.php',
    'SAML2_XML_md_AffiliationDescriptor' => $vendorDir . '/simplesamlphp/saml2/src/SAML2/XML/md/AffiliationDescriptor.php',
    'SAML2_XML_md_AttributeAuthorityDescriptor' => $vendorDir . '/simplesamlphp/saml2/src/SAML2/XML/md/AttributeAuthorityDescriptor.php',
    'SAML2_XML_md_AttributeConsumingService' => $vendorDir . '/simplesamlphp/saml2/src/SAML2/XML/md/AttributeConsumingService.php',
    'SAML2_XML_md_AuthnAuthorityDescriptor' => $vendorDir . '/simplesamlphp/saml2/src/SAML2/XML/md/AuthnAuthorityDescriptor.php',
    'SAML2_XML_md_ContactPerson' => $vendorDir . '/simplesamlphp/saml2/src/SAML2/XML/md/ContactPerson.php',
    'SAML2_XML_md_EndpointType' => $vendorDir . '/simplesamlphp/saml2/src/SAML2/XML/md/EndpointType.php',
    'SAML2_XML_md_EntitiesDescriptor' => $vendorDir . '/simplesamlphp/saml2/src/SAML2/XML/md/EntitiesDescriptor.php',
    'SAML2_XML_md_EntityDescriptor' => $vendorDir . '/simplesamlphp/saml2/src/SAML2/XML/md/EntityDescriptor.php',
    'SAML2_XML_md_Extensions' => $vendorDir . '/simplesamlphp/saml2/src/SAML2/XML/md/Extensions.php',
    'SAML2_XML_md_IDPSSODescriptor' => $vendorDir . '/simplesamlphp/saml2/src/SAML2/XML/md/IDPSSODescriptor.php',
    'SAML2_XML_md_IndexedEndpointType' => $vendorDir . '/simplesamlphp/saml2/src/SAML2/XML/md/IndexedEndpointType.php',
    'SAML2_XML_md_KeyDescriptor' => $vendorDir . '/simplesamlphp/saml2/src/SAML2/XML/md/KeyDescriptor.php',
    'SAML2_XML_md_Organization' => $vendorDir . '/simplesamlphp/saml2/src/SAML2/XML/md/Organization.php',
    'SAML2_XML_md_PDPDescriptor' => $vendorDir . '/simplesamlphp/saml2/src/SAML2/XML/md/PDPDescriptor.php',
    'SAML2_XML_md_RequestedAttribute' => $vendorDir . '/simplesamlphp/saml2/src/SAML2/XML/md/RequestedAttribute.php',
    'SAML2_XML_md_RoleDescriptor' => $vendorDir . '/simplesamlphp/saml2/src/SAML2/XML/md/RoleDescriptor.php',
    'SAML2_XML_md_SPSSODescriptor' => $vendorDir . '/simplesamlphp/saml2/src/SAML2/XML/md/SPSSODescriptor.php',
    'SAML2_XML_md_SSODescriptorType' => $vendorDir . '/simplesamlphp/saml2/src/SAML2/XML/md/SSODescriptorType.php',
    'SAML2_XML_md_UnknownRoleDescriptor' => $vendorDir . '/simplesamlphp/saml2/src/SAML2/XML/md/UnknownRoleDescriptor.php',
    'SAML2_XML_mdattr_EntityAttributes' => $vendorDir . '/simplesamlphp/saml2/src/SAML2/XML/mdattr/EntityAttributes.php',
    'SAML2_XML_mdrpi_Common' => $vendorDir . '/simplesamlphp/saml2/src/SAML2/XML/mdrpi/Common.php',
    'SAML2_XML_mdrpi_PublicationInfo' => $vendorDir . '/simplesamlphp/saml2/src/SAML2/XML/mdrpi/PublicationInfo.php',
    'SAML2_XML_mdrpi_RegistrationInfo' => $vendorDir . '/simplesamlphp/saml2/src/SAML2/XML/mdrpi/RegistrationInfo.php',
    'SAML2_XML_mdui_DiscoHints' => $vendorDir . '/simplesamlphp/saml2/src/SAML2/XML/mdui/DiscoHints.php',
    'SAML2_XML_mdui_Keywords' => $vendorDir . '/simplesamlphp/saml2/src/SAML2/XML/mdui/Keywords.php',
    'SAML2_XML_mdui_Logo' => $vendorDir . '/simplesamlphp/saml2/src/SAML2/XML/mdui/Logo.php',
    'SAML2_XML_mdui_UIInfo' => $vendorDir . '/simplesamlphp/saml2/src/SAML2/XML/mdui/UIInfo.php',
    'SAML2_XML_saml_Attribute' => $vendorDir . '/simplesamlphp/saml2/src/SAML2/XML/saml/Attribute.php',
    'SAML2_XML_saml_AttributeValue' => $vendorDir . '/simplesamlphp/saml2/src/SAML2/XML/saml/AttributeValue.php',
    'SAML2_XML_saml_NameID' => $vendorDir . '/simplesamlphp/saml2/src/SAML2/XML/saml/NameID.php',
    'SAML2_XML_saml_SubjectConfirmation' => $vendorDir . '/simplesamlphp/saml2/src/SAML2/XML/saml/SubjectConfirmation.php',
    'SAML2_XML_saml_SubjectConfirmationData' => $vendorDir . '/simplesamlphp/saml2/src/SAML2/XML/saml/SubjectConfirmationData.php',
    'SAML2_XML_samlp_Extensions' => $vendorDir . '/simplesamlphp/saml2/src/SAML2/XML/samlp/Extensions.php',
    'SAML2_XML_shibmd_Scope' => $vendorDir . '/simplesamlphp/saml2/src/SAML2/XML/shibmd/Scope.php',
    'SimpleSAML\\Database' => $baseDir . '/lib/SimpleSAML/Database.php',
    'SimpleSAML\\Utils\\Arrays' => $baseDir . '/lib/SimpleSAML/Utils/Arrays.php',
    'SimpleSAML\\Utils\\Attributes' => $baseDir . '/lib/SimpleSAML/Utils/Attributes.php',
    'SimpleSAML\\Utils\\Auth' => $baseDir . '/lib/SimpleSAML/Utils/Auth.php',
    'SimpleSAML\\Utils\\Config' => $baseDir . '/lib/SimpleSAML/Utils/Config.php',
    'SimpleSAML\\Utils\\Config\\Metadata' => $baseDir . '/lib/SimpleSAML/Utils/Config/Metadata.php',
    'SimpleSAML\\Utils\\Crypto' => $baseDir . '/lib/SimpleSAML/Utils/Crypto.php',
    'SimpleSAML\\Utils\\HTTP' => $baseDir . '/lib/SimpleSAML/Utils/HTTP.php',
    'SimpleSAML\\Utils\\Net' => $baseDir . '/lib/SimpleSAML/Utils/Net.php',
    'SimpleSAML\\Utils\\Random' => $baseDir . '/lib/SimpleSAML/Utils/Random.php',
    'SimpleSAML\\Utils\\System' => $baseDir . '/lib/SimpleSAML/Utils/System.php',
    'SimpleSAML\\Utils\\Time' => $baseDir . '/lib/SimpleSAML/Utils/Time.php',
    'SimpleSAML\\Utils\\XML' => $baseDir . '/lib/SimpleSAML/Utils/XML.php',
    'SimpleSAML_AuthMemCookie' => $baseDir . '/lib/SimpleSAML/AuthMemCookie.php',
    'SimpleSAML_Auth_Default' => $baseDir . '/lib/SimpleSAML/Auth/Default.php',
    'SimpleSAML_Auth_LDAP' => $baseDir . '/lib/SimpleSAML/Auth/LDAP.php',
    'SimpleSAML_Auth_ProcessingChain' => $baseDir . '/lib/SimpleSAML/Auth/ProcessingChain.php',
    'SimpleSAML_Auth_ProcessingFilter' => $baseDir . '/lib/SimpleSAML/Auth/ProcessingFilter.php',
    'SimpleSAML_Auth_Simple' => $baseDir . '/lib/SimpleSAML/Auth/Simple.php',
    'SimpleSAML_Auth_Source' => $baseDir . '/lib/SimpleSAML/Auth/Source.php',
    'SimpleSAML_Auth_State' => $baseDir . '/lib/SimpleSAML/Auth/State.php',
    'SimpleSAML_Auth_TimeLimitedToken' => $baseDir . '/lib/SimpleSAML/Auth/TimeLimitedToken.php',
    'SimpleSAML_Bindings_Shib13_Artifact' => $baseDir . '/lib/SimpleSAML/Bindings/Shib13/Artifact.php',
    'SimpleSAML_Bindings_Shib13_HTTPPost' => $baseDir . '/lib/SimpleSAML/Bindings/Shib13/HTTPPost.php',
    'SimpleSAML_Configuration' => $baseDir . '/lib/SimpleSAML/Configuration.php',
    'SimpleSAML_Error_Assertion' => $baseDir . '/lib/SimpleSAML/Error/Assertion.php',
    'SimpleSAML_Error_AuthSource' => $baseDir . '/lib/SimpleSAML/Error/AuthSource.php',
    'SimpleSAML_Error_BadRequest' => $baseDir . '/lib/SimpleSAML/Error/BadRequest.php',
    'SimpleSAML_Error_BadUserInnput' => $baseDir . '/lib/SimpleSAML/Error/BadUserInnput.php',
    'SimpleSAML_Error_Error' => $baseDir . '/lib/SimpleSAML/Error/Error.php',
    'SimpleSAML_Error_Exception' => $baseDir . '/lib/SimpleSAML/Error/Exception.php',
    'SimpleSAML_Error_InvalidCredential' => $baseDir . '/lib/SimpleSAML/Error/InvalidCredential.php',
    'SimpleSAML_Error_MetadataNotFound' => $baseDir . '/lib/SimpleSAML/Error/MetadataNotFound.php',
    'SimpleSAML_Error_NoPassive' => $baseDir . '/lib/SimpleSAML/Error/NoPassive.php',
    'SimpleSAML_Error_NoState' => $baseDir . '/lib/SimpleSAML/Error/NoState.php',
    'SimpleSAML_Error_NotFound' => $baseDir . '/lib/SimpleSAML/Error/NotFound.php',
    'SimpleSAML_Error_ProxyCountExceeded' => $baseDir . '/lib/SimpleSAML/Error/ProxyCountExceeded.php',
    'SimpleSAML_Error_UnserializableException' => $baseDir . '/lib/SimpleSAML/Error/UnserializableException.php',
    'SimpleSAML_Error_User' => $baseDir . '/lib/SimpleSAML/Error/User.php',
    'SimpleSAML_Error_UserAborted' => $baseDir . '/lib/SimpleSAML/Error/UserAborted.php',
    'SimpleSAML_Error_UserNotFound' => $baseDir . '/lib/SimpleSAML/Error/UserNotFound.php',
    'SimpleSAML_IdP' => $baseDir . '/lib/SimpleSAML/IdP.php',
    'SimpleSAML_IdP_LogoutHandler' => $baseDir . '/lib/SimpleSAML/IdP/LogoutHandler.php',
    'SimpleSAML_IdP_LogoutIFrame' => $baseDir . '/lib/SimpleSAML/IdP/LogoutIFrame.php',
    'SimpleSAML_IdP_LogoutTraditional' => $baseDir . '/lib/SimpleSAML/IdP/LogoutTraditional.php',
    'SimpleSAML_Logger' => $baseDir . '/lib/SimpleSAML/Logger.php',
    'SimpleSAML_Logger_LoggingHandler' => $baseDir . '/lib/SimpleSAML/Logger/LoggingHandler.php',
    'SimpleSAML_Logger_LoggingHandlerErrorLog' => $baseDir . '/lib/SimpleSAML/Logger/LoggingHandlerErrorLog.php',
    'SimpleSAML_Logger_LoggingHandlerFile' => $baseDir . '/lib/SimpleSAML/Logger/LoggingHandlerFile.php',
    'SimpleSAML_Logger_LoggingHandlerSyslog' => $baseDir . '/lib/SimpleSAML/Logger/LoggingHandlerSyslog.php',
    'SimpleSAML_Memcache' => $baseDir . '/lib/SimpleSAML/Memcache.php',
    'SimpleSAML_Metadata_MetaDataStorageHandler' => $baseDir . '/lib/SimpleSAML/Metadata/MetaDataStorageHandler.php',
    'SimpleSAML_Metadata_MetaDataStorageHandlerFlatFile' => $baseDir . '/lib/SimpleSAML/Metadata/MetaDataStorageHandlerFlatFile.php',
    'SimpleSAML_Metadata_MetaDataStorageHandlerMDX' => $baseDir . '/lib/SimpleSAML/Metadata/MetaDataStorageHandlerMDX.php',
    'SimpleSAML_Metadata_MetaDataStorageHandlerPdo' => $baseDir . '/lib/SimpleSAML/Metadata/MetaDataStorageHandlerPdo.php',
    'SimpleSAML_Metadata_MetaDataStorageHandlerSerialize' => $baseDir . '/lib/SimpleSAML/Metadata/MetaDataStorageHandlerSerialize.php',
    'SimpleSAML_Metadata_MetaDataStorageHandlerXML' => $baseDir . '/lib/SimpleSAML/Metadata/MetaDataStorageHandlerXML.php',
    'SimpleSAML_Metadata_MetaDataStorageSource' => $baseDir . '/lib/SimpleSAML/Metadata/MetaDataStorageSource.php',
    'SimpleSAML_Metadata_SAMLBuilder' => $baseDir . '/lib/SimpleSAML/Metadata/SAMLBuilder.php',
    'SimpleSAML_Metadata_SAMLParser' => $baseDir . '/lib/SimpleSAML/Metadata/SAMLParser.php',
    'SimpleSAML_Metadata_Signer' => $baseDir . '/lib/SimpleSAML/Metadata/Signer.php',
    'SimpleSAML_Module' => $baseDir . '/lib/SimpleSAML/Module.php',
    'SimpleSAML_Session' => $baseDir . '/lib/SimpleSAML/Session.php',
    'SimpleSAML_SessionHandler' => $baseDir . '/lib/SimpleSAML/SessionHandler.php',
    'SimpleSAML_SessionHandlerCookie' => $baseDir . '/lib/SimpleSAML/SessionHandlerCookie.php',
    'SimpleSAML_SessionHandlerPHP' => $baseDir . '/lib/SimpleSAML/SessionHandlerPHP.php',
    'SimpleSAML_SessionHandlerStore' => $baseDir . '/lib/SimpleSAML/SessionHandlerStore.php',
    'SimpleSAML_Stats' => $baseDir . '/lib/SimpleSAML/Stats.php',
    'SimpleSAML_Stats_Output' => $baseDir . '/lib/SimpleSAML/Stats/Output.php',
    'SimpleSAML_Store' => $baseDir . '/lib/SimpleSAML/Store.php',
    'SimpleSAML_Store_Memcache' => $baseDir . '/lib/SimpleSAML/Store/Memcache.php',
    'SimpleSAML_Store_SQL' => $baseDir . '/lib/SimpleSAML/Store/SQL.php',
    'SimpleSAML_Utilities' => $baseDir . '/lib/SimpleSAML/Utilities.php',
    'SimpleSAML_XHTML_EMail' => $baseDir . '/lib/SimpleSAML/XHTML/EMail.php',
    'SimpleSAML_XHTML_IdPDisco' => $baseDir . '/lib/SimpleSAML/XHTML/IdPDisco.php',
    'SimpleSAML_XHTML_Template' => $baseDir . '/lib/SimpleSAML/XHTML/Template.php',
    'SimpleSAML_XML_Errors' => $baseDir . '/lib/SimpleSAML/XML/Errors.php',
    'SimpleSAML_XML_Parser' => $baseDir . '/lib/SimpleSAML/XML/Parser.php',
    'SimpleSAML_XML_Shib13_AuthnRequest' => $baseDir . '/lib/SimpleSAML/XML/Shib13/AuthnRequest.php',
    'SimpleSAML_XML_Shib13_AuthnResponse' => $baseDir . '/lib/SimpleSAML/XML/Shib13/AuthnResponse.php',
    'SimpleSAML_XML_Signer' => $baseDir . '/lib/SimpleSAML/XML/Signer.php',
    'SimpleSAML_XML_Validator' => $baseDir . '/lib/SimpleSAML/XML/Validator.php',
    'SimpleSamlPhp\\Composer\\ModuleInstaller' => $vendorDir . '/simplesamlphp/composer-module-installer/src/SimpleSamlPhp/Composer/ModuleInstaller.php',
    'SimpleSamlPhp\\Composer\\ModuleInstallerPlugin' => $vendorDir . '/simplesamlphp/composer-module-installer/src/SimpleSamlPhp/Composer/ModuleInstallerPlugin.php',
    'WhiteHat101\\Crypt\\APR1_MD5' => $vendorDir . '/whitehat101/apr1-md5/src/APR1_MD5.php',
    'XMLSecEnc' => $vendorDir . '/robrichards/xmlseclibs/src/XMLSecEnc.php',
    'XMLSecurityDSig' => $vendorDir . '/robrichards/xmlseclibs/src/XMLSecurityDSig.php',
    'XMLSecurityKey' => $vendorDir . '/robrichards/xmlseclibs/src/XMLSecurityKey.php',
);
