<?php

/*
* AUTHOR: Samuel Muñoz Hidalgo
* EMAIL: samuel.mh@gmail.com
* LAST REVISION: 16-DEC-08
* DESCRIPTION: 'login-infocard' module dictionary.
*/

$lang = array(
	'wrong_IC' => array (
		'no' => '',
		'nn' => '',
		'da' => '',
		'en' => 'Invalid InfoCard',
		'de' => '',
		'sv' => '',
		'fi' => '',
		'es' => 'InfoCard errónea',
		'fr' => '',
		'nl' => '',
		'lb' => '',
		'sl' => '',
		'hr' => '',
		'hu' => '',
		'pt' => '',
		'pt-br' => '',
	),
	'get_IC' => array (
		'no' => '',
		'nn' => '',
		'da' => '',
		'en' => 'Get your InfoCard',
		'de' => '',
		'sv' => '',
		'fi' => '',
		'es' => 'Consiga su InfoCard',
		'fr' => '',
		'nl' => '',
		'lb' => '',
		'sl' => '',
		'hr' => '',
		'hu' => '',
		'pt' => '',
		'pt-br' => '',
	),
	'get_IC_link' => array (
		'no' => '',
		'nn' => '',
		'da' => '',
		'en' => 'Click here to get your InfoCard',
		'de' => '',
		'sv' => '',
		'fi' => '',
		'es' => 'Pinche aquí para conseguir su InfoCard',
		'fr' => '',
		'nl' => '',
		'lb' => '',
		'sl' => '',
		'hr' => '',
		'hu' => '',
		'pt' => '',
		'pt-br' => '',
	),
	'form_username' => array (
		'no' => '',
		'nn' => '',
		'da' => '',
		'en' => 'Username',
		'de' => '',
		'sv' => '',
		'fi' => '',
		'es' => 'Usuario',
		'fr' => '',
		'nl' => '',
		'lb' => '',
		'sl' => '',
		'hr' => '',
		'hu' => '',
		'pt' => '',
		'pt-br' => '',
	),
	'form_password' => array (
		'no' => '',
		'nn' => '',
		'da' => '',
		'en' => 'Password',
		'de' => '',
		'sv' => '',
		'fi' => '',
		'es' => 'Contraseña',
		'fr' => '',
		'nl' => '',
		'lb' => '',
		'sl' => '',
		'hr' => '',
		'hu' => '',
		'pt' => '',
		'pt-br' => '',
	),
	'error_header' => array (
		'no' => 'Feil',
		'nn' => 'Feil',
		'da' => 'Fejl',
		'en' => 'Error',
		'de' => 'Fehler',
		'sv' => 'Fel',
		'fi' => 'Virhe',
		'es' => 'Error',
		'fr' => 'Erreur',
		'nl' => 'Fout',
		'lb' => 'Fehler',
		'sl' => 'Napaka',
		'hr' => 'Greška',
		'hu' => 'Hiba',
		'pt' => 'Erro',
		'pt-br' => 'Erro',
	),
	'user_IC_header' => array (
		'no' => '',
		'nn' => '',
		'da' => '',
		'en' => 'Select an InfoCard',
		'de' => '',
		'sv' => '',
		'fi' => '',
		'es' => 'Seleccione una InfoCard',
		'fr' => '',
		'nl' => '',
		'lb' => '',
		'sl' => '',
		'hr' => '',
		'hu' => '',
		'pt' => '',
		'pt-br' => '',
	),
	'user_IC_text' => array (
		'no' => '',
		'nn' => '',
		'da' => '',
		'en' => 'A service has requested you to authenticate yourself. Please click on the image below to start a session with an InfoCard.',
		'de' => '',
		'sv' => '',
		'fi' => '',
		'es' => 'Un servicio solicita que se autentique. Pinche en la imagen inferior para iniciar una sesión con una InfoCard.',
		'fr' => '',
		'nl' => '',
		'lb' => '',
		'sl' => '',
		'hr' => '',
		'hu' => '',
		'pt' => '',
		'pt-br' => '',
	),
	'login_button' => array (
		'no' => '',
		'nn' => '',
		'da' => '',
		'en' => ' ',
		'de' => '',
		'sv' => '',
		'fi' => '',
		'es' => ' ',
		'fr' => '',
		'nl' => '',
		'lb' => '',
		'sl' => '',
		'hr' => '',
		'hu' => '',
		'pt' => '',
		'pt-br' => '',
	),
	'get_button' => array (
		'no' => '',
		'nn' => '',
		'da' => '',
		'en' => 'Get my Infocard',
		'de' => '',
		'sv' => '',
		'fi' => '',
		'es' => 'Conseguir mi Infocard',
		'fr' => '',
		'nl' => '',
		'lb' => '',
		'sl' => '',
		'hr' => '',
		'hu' => '',
		'pt' => '',
		'pt-br' => '',
	),
	'help_header' => array (
		'no' => '',
		'nn' => '',
		'da' => '',
		'en' => 'HELP! What is an InfoCard?',
		'de' => '',
		'sv' => '',
		'fi' => '',
		'es' => '¡Ayuda! ¿Qué es una InfoCard?',
		'fr' => '',
		'nl' => '',
		'lb' => '',
		'sl' => '',
		'hr' => '',
		'hu' => '',
		'pt' => '',
		'pt-br' => '',
	),
	'help_text' => array (
		'no' => '',
		'nn' => '',
		'da' => '',
		'en' => 'Information Cards (aka InfoCard) is a web authentication technology. Contact with your services provider in order to configure your computer and give you and Information Card (identification virtual card).',
		'de' => '',
		'sv' => '',
		'fi' => '',
		'es' => 'Information Cards (alias Infocard) es una tecnología de autenticación web. Consulte con su proveedor de servicios para que le ayude a configurar su máquina y le expida una Information Card (tarjeta virtual de identificación).',
		'fr' => '',
		'nl' => '',
		'lb' => '',
		'sl' => '',
		'hr' => '',
		'hu' => '',
		'pt' => '',
		'pt-br' => '',
	),
	'help_desk_link' => array (
		'no' => 'Hjemmesiden til brukerstøtte',
		'nn' => 'Heimeside for brukarstøtte',
		'da' => 'Servicedesk',
		'en' => 'Help desk homepage',
		'de' => 'Seite des Helpdesk',
		'sv' => 'Hemsida för helpdesk',
		'es' => 'Página de soporte técnico',
		'nl' => 'Helpdesk homepage',
		'sl' => 'Spletna stran tehnične podpore uporabnikom.',
		'hr' => 'Helpdesk stranice',
		'hu' => 'Ügyfélszolgálat weboldala',
		'pt' => 'Página do serviço de apoio ao utilizador',
		'pt-br' => 'Central de Ajuda',
	),
	'help_desk_email' => array (
		'no' => 'Send e-post til brukerstøtte',
		'nn' => 'Send epost til brukarstøtte',
		'da' => 'Send en e-mail servicedesk',
		'en' => 'Send e-mail to help desk',
		'de' => 'Email an den Helpdesk senden',
		'sv' => 'Skicka e-post till helpdesk',
		'es' => 'Enviar correo electrónico al soporte técnico',
		'nl' => 'Stuur een e-mail naar de helpdesk',
		'sl' => 'Pošlji e-poštno sporočilo tehnični podpori.',
		'hr' => 'Pošaljite e-mail helpdesk službi',
		'hu' => 'Küldjön e-mailt az ügyfélszolgálatnak',
		'pt' => 'Enviar um e-mail para o serviço de apoio ao utilizador',
		'pt-br' => 'Envie um e-mail para a Central de Ajuda.',
	),
	'contact_info' => array (
		'no' => 'Kontaktinformasjon',
		'nn' => 'Kontaktinformasjon',
		'da' => 'Kontaktoplysninger',
		'en' => 'Contact information',
		'de' => 'Kontakt',
		'sv' => 'Kontaktinformation',
		'es' => 'Información de contacto',
		'nl' => 'Contact informatie',
		'sl' => 'Kontakt',
		'hr' => 'Kontakt podaci',
		'hu' => 'Elérési információk',
		'pt' => 'Contactos',
		'pt-br' => 'Informações de Contato',
	),	
	'getcardform_title' => array (
		'no' => '',
		'nn' => '',
		'da' => '',
		'en' => 'Authentication form',
		'de' => '',
		'sv' => '',
		'fi' => '',
		'es' => 'Formulario de autenticación',
		'fr' => '',
		'nl' => '',
		'lb' => '',
		'sl' => '',
		'hr' => '',
		'hu' => '',
		'pt' => '',
		'pt-br' => '',
	),
	'getcardform_self_title' => array (
		'no' => '',
		'nn' => '',
		'da' => '',
		'en' => 'Self-Issued Card form',
		'de' => '',
		'sv' => '',
		'fi' => '',
		'es' => 'Formulario de tarjeta autogestionada',
		'fr' => '',
		'nl' => '',
		'lb' => '',
		'sl' => '',
		'hr' => '',
		'hu' => '',
		'pt' => '',
		'pt-br' => '',
	),
	'getcardform_self_text' => array (
		'no' => '',
		'nn' => '',
		'da' => '',
		'en' => 'Please, enter a self issued InfoCard in order to link it with the managed one we are generating.',
		'de' => '',
		'sv' => '',
		'fi' => '',
		'es' => 'Por favor, introduzca una InfoCard autogestionada para ligarla a la que vamos a generarle.',
		'fr' => '',
		'nl' => '',
		'lb' => '',
		'sl' => '',
		'hr' => '',
		'hu' => '',
		'pt' => '',
		'pt-br' => '',
	),
	'getcardform_finished_title' => array (
		'no' => '',
		'nn' => '',
		'da' => '',
		'en' => 'Congratulations, your Infocard was succesfully generated',
		'de' => '',
		'sv' => '',
		'fi' => '',
		'es' => 'Enhorabuena, generó su Infocard con éxito',
		'fr' => '',
		'nl' => '',
		'lb' => '',
		'sl' => '',
		'hr' => '',
		'hu' => '',
		'pt' => '',
		'pt-br' => '',
	),
	'getcardform_finished_text' => array (
		'no' => '',
		'nn' => '',
		'da' => '',
		'en' => 'Please follow the next link to get to the login page.',
		'de' => '',
		'sv' => '',
		'fi' => '',
		'es' => 'Siga el siguiente enlace para ir a la página de login',
		'fr' => '',
		'nl' => '',
		'lb' => '',
		'sl' => '',
		'hr' => '',
		'hu' => '',
		'pt' => '',
		'pt-br' => '',
	),
	'NO_password' => array (
		'no' => '',
		'nn' => '',
		'da' => '',
		'en' => 'Please fill in your password',
		'de' => '',
		'sv' => '',
		'fi' => '',
		'es' => 'Introduce una contraseña',
		'fr' => '',
		'nl' => '',
		'lb' => '',
		'sl' => '',
		'hr' => '',
		'hu' => '',
		'pt' => '',
		'pt-br' => '',
	),
	'NO_user' => array (
		'no' => '',
		'nn' => '',
		'da' => '',
		'en' => 'Please fill in your username',
		'de' => '',
		'sv' => '',
		'fi' => '',
		'es' => 'Introduce tu nombre de usuario',
		'fr' => '',
		'nl' => '',
		'lb' => '',
		'sl' => '',
		'hr' => '',
		'hu' => '',
		'pt' => '',
		'pt-br' => '',
	),
	'Wrong_user_pass' => array (
		'no' => '',
		'nn' => '',
		'da' => '',
		'en' => 'Wrong username or password',
		'de' => '',
		'sv' => '',
		'fi' => '',
		'es' => 'Usuario o contraseña inválidos ',
		'fr' => '',
		'nl' => '',
		'lb' => '',
		'sl' => '',
		'hr' => '',
		'hu' => '',
		'pt' => '',
		'pt-br' => '',
	),
	'' => array (
		'no' => '',
		'nn' => '',
		'da' => '',
		'en' => '',
		'de' => '',
		'sv' => '',
		'fi' => '',
		'es' => '',
		'fr' => '',
		'nl' => '',
		'lb' => '',
		'sl' => '',
		'hr' => '',
		'hu' => '',
		'pt' => '',
		'pt-br' => '',
	),
	'' => array (
		'no' => '',
		'nn' => '',
		'da' => '',
		'en' => '',
		'de' => '',
		'sv' => '',
		'fi' => '',
		'es' => '',
		'fr' => '',
		'nl' => '',
		'lb' => '',
		'sl' => '',
		'hr' => '',
		'hu' => '',
		'pt' => '',
		'pt-br' => '',
	),

);
