<?php

// autoload_namespaces.php @generated by Composer

$vendorDir = dirname(dirname(__FILE__));
$baseDir = dirname($vendorDir);

return array(
    'SimpleSamlPhp\\Composer' => array($vendorDir . '/simplesamlphp/composer-module-installer/src'),
    'SimpleSAML' => array($baseDir . '/lib'),
    'SAML2_' => array($vendorDir . '/simplesamlphp/saml2/src'),
    'Psr\\Log\\' => array($vendorDir . '/psr/log'),
);
