<?php
/**
 * WS-Federation remote IdP metadata for SimpleSAMLphp.
 */

$metadata['urn:federation:pingfederate:localhost'] = array(
	'prp' => 'https://localhost:9031/idp/prp.wsf',
	'certificate' => 'pingfed-localhost.pem',
);
