<?php

/*
 * DO NOT EDIT THIS FILE MANUALLY.
 * If you want to contribute to translation, you must instead use the 
 * SimpleSAMLphp translation portal:
 * https://translation.rnd.feide.no/index.php?aid=simplesamlphp
 *
 * This file is automatically generated from the translation portal.
 */

$lang = array(
	'yes' => array (
		'no' => 'Ja, fortsett',
		'nn' => 'Ja, fortsett',
		'da' => 'Ja, jeg accepterer',
		'en' => 'Yes, continue',
		'de' => 'Ja, ich stimmte zu',
		'sv' => 'Ja',
		'fi' => 'Kyllä',
		'es' => 'Sí',
		'fr' => 'Oui',
		'nl' => 'Ja, ik ga akkoord',
		'lb' => 'Jo',
		'sl' => 'Da, nadaljuj',
		'hr' => 'Da, prihvaćam',
		'hu' => 'Igen, elfogadom',
		'pl' => 'Tak, akceptuję',
		'pt' => 'Sim, Aceito',
		'pt-br' => 'Sim, Aceito',
		'tr' => 'Evet, devam et',
	),
	'no' => array (
		'no' => 'Nei, avbryt',
		'nn' => 'Nei, avbryt',
		'da' => 'Nej, jeg accepterer ikke',
		'en' => 'No, cancel',
		'de' => 'Nein, ich stimmte nicht zu',
		'sv' => 'Nej',
		'fi' => 'ei',
		'es' => 'No',
		'fr' => 'Non',
		'nl' => 'Nee, ik weiger',
		'lb' => 'Nee',
		'sl' => 'Ne, prekliči',
		'hr' => 'Ne privaćam',
		'hu' => 'Nem, nem fogadom el',
		'pl' => 'Nie, nie akceptuję',
		'pt' => 'Não aceito',
		'pt-br' => 'Não, não aceito',
		'tr' => 'Hayır, iptal et',
	),
	'remember' => array (
		'no' => 'Godta også for fremtiden',
		'nn' => 'Godta også for framtida',
		'da' => 'Husk samtykke',
		'en' => 'Remember',
		'de' => 'Zustimmung merken',
		'sv' => 'Spara samtycke',
		'fi' => 'Muista',
		'es' => 'Recordar el consentimiento',
		'fr' => 'Se souvenir du consentement',
		'nl' => 'Bewaar toestemming',
		'lb' => 'Zoustëmmung verhalen',
		'sl' => 'Zapomni si privolitev.',
		'hr' => 'Zapamti dozvole',
		'hu' => 'Emlékezzen a hozzájárulásra',
		'pl' => 'Pamiętaj moją zgodę',
		'pt' => 'Lembrar a minha escolha',
		'pt-br' => 'Lembrar Consentimento',
		'tr' => 'Hatırla',
	),
	'consent_header' => array (
		'no' => 'Samtykke om overføring av personinformasjon',
		'nn' => 'Samtykke til overføring av personinformasjon',
		'da' => 'Samtykke til at frigive personlige oplysninger',
		'en' => 'Consent about releasing personal information',
		'de' => 'Zustimmung zur Weitergabe persönlicher Daten  ',
		'sv' => 'Samtycke gällande överföring av personinformation',
		'fi' => 'Henkilötietojen luovutuksen hyväksyntä',
		'es' => 'Consentimiento para la liberación de información personal',
		'nl' => 'Toestemming voor het vrijgeven van persoonsgegevens',
		'sl' => 'Odločitev o privolitvi posredovanja vaših osebnih podatkov',
		'hr' => 'Dozvola za isporuku osobnih podataka',
		'hu' => 'Hozzájárulás személyes adatok kiadásához',
		'pl' => 'Zgoda na wysłanie danych osobistych',
		'pt' => 'Consentimento do envio de informação pessoal',
		'tr' => 'Kişisel bilgilerin verilmesi hakkında onay',
	),
	'consent_accept' => array (
		'no' => 'For å fullføre innloggingen må du godta at opplysningene nedenfor sendes til SPNAME.',
		'nn' => 'For å fullføra innlogginga må du godta at opplysningane under blir sende til SPNAME',
		'da' => 'SPNAME kræver at nedenstående oplysninger overføres fra IDPNAME. Vil du acceptere dette?',
		'en' => 'SPNAME requires that the information below is transferred.',
		'de' => 'SPNAME erfordert die Übertragung untenstehender Information von IDPNAME. Akzeptieren Sie das?',
		'sv' => 'Du är på väg att logga in i tjänsten SPNAME. Tjänsten kräver att informationen nedan skickas från IDPNAME. Är detta okej?',
		'fi' => 'Olet kirjautumassa palveluun SPNAME. Identiteetilähteesi henkilötietojasi palvelun tarjoajalle. Hyväksytkö tietojen siirron?',
		'es' => 'Está a punto de acceder al servicio SPNAME. El servicio requiere que la información que se muestra a continuación sea transferida desde IDPNAME. ¿Acepta esto?  ',
		'fr' => 'Vous êtes sur le point de vous connecter au service SPNAME. Lors de l\'ouverture de session, le fournisseur d\'identité enverra des informations sur votre identité à ce service. Acceptez-vous cela ?',
		'nl' => 'U gaat inloggen bij een dienst SPNAME. Tijdens het loginproces stuurt de identity provider zgn. attributen met daarin informatie over uw identiteit voor deze dienst. Bent u het daarmee eens?',
		'lb' => 'Daer sid dobai aerch um service unzemellen SPNAME. Waerend dem Login Prozess schéckt den Identity Provider Attributer, déi Informatiounen iwert aer Identitéit enthaalen. Akzeptéier daer daat?',
		'sl' => 'Pravkar se nameravate prijaviti v storitev SPNAME. Med postopkom prijave bo IdP tej storitvi posredoval atribute, ki vsebujejo informacije o vaši identiteti. Ali se s tem strinjate? ',
		'hr' => 'U tijeku je proces prijave za pristup servisu SPNAME. Servis zahtjeva da IDPNAME isporuči dolje navedene podatke. Slažete li se s time?',
		'hu' => 'Ön azonosítja magát ehhez a szolgáltatáshoz SPNAME. Az azonosítás során IDPNAME az alábbi adatokat fogja küldeni a szolgáltatásnak. Engedélyezi?',
		'pl' => 'SPNAME wymaga aby poniższa informacja została przesłana.',
		'pt' => 'O serviço SPNAME necessita que a informação apresentada em baixo seja transferida.',
		'pt-br' => 'Você está prestes a acessar o serviço SPNAME. O serviço exige que as informações a seguir sejam transferidas do IDPNAME. Você aceita isso?',
		'tr' => 'SPNAME aşağıdaki bilgilerin gönderilmesine ihtiyaç duyuyor.',
	),
	'consent_purpose' => array (
		'no' => 'Formålet med SPNAME er SPDESC',
		'nn' => 'Hensikta med SPNAME er SPDESC',
		'da' => 'SPNAME har til formål at SPDESC',
		'en' => 'The purpose of SPNAME is SPDESC',
		'de' => 'Der Zweck von SPNAME ist SPDESC',
		'sv' => 'Syftet med SPNAME är SPDESC',
		'fi' => 'Palvelun SPNAME käyttötarkoitus on SPDESC',
		'es' => 'El propósito de SPNAME es SPDESC',
		'nl' => 'Het doel van SPNAME is SPDESC',
		'sl' => 'Namen %SPNAME%: %SPDESC%',
		'hr' => 'Svrha SPNAME je SPDESC',
		'hu' => 'A szolgáltatás (SPNAME) ezt a célt szolgálja: SPDESC',
		'pl' => 'Celem SPNAME jest SPDESC',
		'pt' => 'O propósito de SPNAME é SPDESC',
		'tr' => 'SPNAME\'in amacı SPDESC\'tir',
	),
	'consent_privacypolicy' => array (
		'no' => 'Personvern for tjenesten',
		'nn' => 'Personvern for tenesta',
		'da' => 'Tjenestens politik vedrørende personoplysninger',
		'en' => 'Privacypolicy for the service',
		'de' => 'Datenschutzrichtlinie des Dienstes',
		'sv' => 'Tjänstens policy för personlig integritet',
		'fi' => 'Tietosuojaseloste palvelulle',
		'es' => 'Política de privacidad para el servicio',
		'nl' => 'Privacybeleid voor de dienst',
		'sl' => 'Politika zasebnosti za ta SP',
		'hr' => 'Politika zaštite privatnosti',
		'hu' => 'A szolgáltatás adatvádelmi nyilatkozata',
		'pl' => 'Polityka prywatności dla serwisu',
		'pt' => 'Política de privacidade do serviço',
		'pt-br' => 'Política de Privacidade deste serviço',
		'tr' => 'Servis için gizlilik politikası',
	),
	'consent_attributes_header' => array (
		'no' => 'Opplysninger som vil bli sendt til SPNAME',
		'nn' => 'Opplysningar som blir sende til SPNAME',
		'da' => 'Attributter som bliver sendt til SPNAME',
		'en' => 'Information that will be sent to SPNAME',
		'de' => 'Informationen, die an SPNAME gesandt werden',
		'sv' => 'Attribut som kommer att skickas till tjänsten',
		'fi' => 'Tiedot lähetetään palvelulle SPNAME',
		'es' => 'Atributos que serán enviados al servicio',
		'nl' => 'Informatie die naar SPNAME zal worden gestuurd',
		'sl' => 'Atributi, ki bodo poslani SPju',
		'hr' => 'Atributi koji će biti poslani servisu',
		'hu' => 'A(z) SPNAME szolgáltatónak küldött adatok',
		'pl' => 'Atrybuty, które zostaną przesłane do serwisu',
		'pt' => 'Informação que irá ser enviada para SPNAME',
		'tr' => 'SPNAME\'e gönderilecek bilgiler',
	),
	'show_attributes' => array (
		'no' => 'Vis opplysninger',
		'nn' => 'Vis opplysingar',
		'da' => 'Vis attributter',
		'en' => 'Show attributes',
		'de' => 'Attribute zeigen',
		'sv' => 'Visa attribut',
		'fi' => 'Näytä henkilötiedot',
		'es' => 'Mostrar atributos',
		'nl' => 'Toon attributen',
		'sl' => 'Prikaži atribute',
		'hr' => 'Prikaži atribute',
		'hu' => 'Mutasd az attribútumokat',
		'pl' => 'Wyświetl atrybuty',
		'pt' => 'Mostrar atributos',
		'tr' => 'Özellikleri göster',
	),
	'show_attribute' => array (
		'no' => 'Vis innhold',
		'nn' => 'Vis innhald',
		'da' => 'Vis indhold',
		'en' => 'Show content',
		'sv' => 'Visa samtycke',
		'es' => 'Mostrart consentimiento',
		'nl' => 'Toon inhoud',
		'sl' => 'Prikaži vsebino',
	),
	'login' => array (
		'no' => 'innlogging',
		'nn' => 'Logg inn',
		'da' => 'login',
		'en' => 'login',
		'de' => 'anmelden',
		'sv' => 'Logga in',
		'fi' => 'Tunnus',
		'es' => 'login',
		'fr' => 'ouvrir une session',
		'nl' => 'Login',
		'lb' => 'anloggen',
		'sl' => 'Prijava',
		'hr' => 'prijava',
		'hu' => 'bejelentkezés',
		'pl' => 'login',
		'pt' => 'Entrar',
		'pt-br' => 'login',
		'tr' => 'Giriş',
	),
	'service_providers_for' => array (
		'no' => 'Tjenesteleverandør for',
		'nn' => 'Tenesteleverandørar for',
		'da' => 'Tjenesteudbyder for',
		'en' => 'Service Providers for',
		'de' => 'Service-Provider für',
		'sv' => 'Tjänsteleverantörer för',
		'fi' => 'Palveluntarjoaja',
		'es' => 'Proveedores de servicio para',
		'fr' => 'Fournisseurs de services pour',
		'nl' => 'Service Providers voor',
		'lb' => 'Service Provider fir',
		'sl' => 'SP za',
		'hr' => 'Davatelji usluge za',
		'hu' => 'Alkalmazásszolgáltatók a következő számára',
		'pl' => 'Dostawca Serwisu dla',
		'pt' => 'Fornecedores de Serviço (SP) para',
		'pt-br' => 'Provedor de serviços para',
		'tr' => 'için Servis Sağlayıcılar',
	),
	'service_provider_header' => array (
		'no' => 'Tjenesteleverandør',
		'nn' => 'Tenesteleverandør',
		'da' => 'Tjenesteudbyder',
		'en' => 'Service Provider',
		'de' => 'Service-Provider',
		'sv' => 'Tjänsteleverantör',
		'fi' => 'Palveluntarjoaja',
		'es' => 'Proveedor de servicio',
		'fr' => 'Fournisseur de service',
		'nl' => 'Service Provider',
		'lb' => 'Service Provider',
		'sl' => 'SP',
		'hr' => 'Davatelj usluge',
		'hu' => 'Alkalmazásszolgáltató',
		'pl' => 'Dostawca serwisu',
		'pt' => 'Fornecedor de Serviço (SP)',
		'pt-br' => 'Provedor de Serviços',
		'tr' => 'Servis Sağlayıcı',
	),
	'status_header' => array (
		'no' => 'Samtykke-status',
		'nn' => 'Samtykkestatus',
		'da' => 'Samtykke status',
		'en' => 'Consent status',
		'de' => 'Zustimmungsstatus',
		'sv' => 'Status för samtycke',
		'fi' => 'Hyväksynnään tila',
		'es' => 'Estado del consentimiento',
		'fr' => 'État des consentements',
		'nl' => 'Toestemming status',
		'lb' => 'Zoustëmmungsstatus',
		'sl' => 'Stanje privolitve',
		'hr' => 'Status dozvole',
		'hu' => 'Hozzájárulás állapota',
		'pl' => 'Status zgody',
		'pt' => 'Consentimento',
		'pt-br' => 'Status do Consentimento',
		'tr' => 'Onay durumu',
	),
	'show_hide_attributes' => array (
		'no' => 'Vis/skjul opplysninger',
		'nn' => 'Vis/skjul opplysningar',
		'da' => 'vis/skjul attributter',
		'en' => 'show/hide attributes',
		'de' => 'zeige/verstecke Eigenschaften',
		'sv' => 'visa/göm attribut',
		'fi' => 'Näytä/piilota attribuutteja',
		'es' => 'Mostrar/ocultar atributos',
		'fr' => 'montrer/cacher les attributs',
		'nl' => 'toon/verberg attributen',
		'lb' => 'Attributer weisen/verstoppen',
		'sl' => 'prikaži/skrij atribute',
		'hr' => 'prikaži/sakrij atribute',
		'hu' => 'attribútumok mutatása/elrejtése',
		'pl' => 'pokaż/ukryj atrybuty',
		'pt' => 'Mostrar/Ocultar atributos',
		'pt-br' => 'mostra/esconder Atributos',
		'tr' => 'bilgileri göster/gizle ',
	),
	'noconsent_title' => array (
		'no' => 'Ikke akseptert overføring av informasjon',
		'nn' => 'Ikkje akseptert overføring av informasjon',
		'da' => 'Manglende samtykke',
		'en' => 'No consent given',
		'de' => 'Zustimmung verweigert',
		'sv' => 'Inget samtycket givet',
		'fi' => 'Lupaa ei annettu',
		'es' => 'No se dió el consentimiento',
		'nl' => 'Geen toestemming gegeven',
		'sl' => 'Privolitev ni bila dana.',
		'hr' => 'Nema pristanka',
		'hu' => 'Nincs hozzájárulás',
		'pl' => 'Nie wyrażono zgody',
		'pt' => 'Consentimento negado',
		'pt-br' => 'Nenhum consentimento dado',
		'tr' => 'Onay verilmemiş',
	),
	'noconsent_text' => array (
		'no' => 'Du har ikke akseptert å overlevere opplysninger til tjenesteleverandøren.',
		'nn' => 'Du har ikkje akseptert til at dine opplysningar kan sendast til tenesteleverandøren',
		'da' => 'Du har ikke givet samtykke til overleveringen af oplysninger til tjenesten',
		'en' => 'You did not give consent for transfering your attributes to the service provider.',
		'de' => 'Sie haben der Weitergabe ihrer Daten an den Service Provider nicht zugestimmt.',
		'sv' => 'Du gav inte samtycke för att överföra dina attribut till tjänsteleverantören.',
		'fi' => 'Et antanut lupaa siirtää henkilötietojasi palveluntarjoajalle',
		'es' => 'No ha dado su consentimiento para tranferir sus atributos al proveedor de servicio.',
		'nl' => 'U heeft geen toestemming gegeven om uw attributen naar de Service Provider te versturen',
		'sl' => 'Niste podali privolitve za posredovanje atributov SP-ju.',
		'hr' => 'Niste dali pristanak da se vaši podaci isporuče davatelju usluge.',
		'hu' => 'Nem adta hozzájárulását, hogy adatait továbbadjuk a szolgáltatónak.',
		'pl' => 'Nie wyraziłeś zgody na przesłanie Twoich atrybutów do Dostawcy Serwisu.',
		'pt' => 'Negou o consentimento para a transferência dos seus atributos para o fornecedor de serviço.',
		'pt-br' => 'Você não deu o consentimento para a transferência de seus atributos para o provedor de serviços.',
		'tr' => 'Bilgilerinizin servis sağlayıcıya gönderilmesi için onay vermediniz.',
	),
	'noconsent_return' => array (
		'no' => 'Gå tilbake til aksept-siden med opplysninger',
		'nn' => 'Gå tilbake til aksept-sida for overføring av opplysningar',
		'da' => 'Gå tilbage',
		'en' => 'Return to consent page',
		'de' => 'Zurück',
		'sv' => 'Åter till sidan för samtycke',
		'fi' => 'Palaa hyväksyntäsivulle',
		'es' => 'Volver a la página de consentimiento',
		'nl' => 'Keer terug naar de toestemmingspagina',
		'sl' => 'Vrnitev na privolitveno stran',
		'hr' => 'Povratak na stranicu s dozvolama',
		'hu' => 'Vissza az hozzájárulás-kezelő oldalra',
		'pl' => 'Powrót do strony wydania zezwolenia.',
		'pt' => 'Voltar à página de consentimento',
		'pt-br' => 'Retornar a página de consentimento',
		'tr' => 'Onay sayfasına geri dön',
	),
	'noconsent_goto_about' => array (
		'no' => 'Gå til informasjonsside om tjenesten',
		'nn' => 'Gå til informasjonssida for tenesta',
		'da' => 'Gå til side med information om tjenesten',
		'en' => 'Go to information page for the service',
		'de' => 'Gehe zur Informationsseite dieses Dienstes',
		'sv' => 'Gå till tjänstens informationssida',
		'fi' => 'Siirry palvelun tiedot -sivulle',
		'es' => 'Ir a la página de información del servicio',
		'nl' => 'Ga naar de informatiepagina voor de service',
		'sl' => 'Pojdi na spletno stran z informacijami o storitvi',
		'hr' => 'Idi na stranicu s informacijama o servisu',
		'hu' => 'A szolgáltatás információs oldalára',
		'pl' => 'Przejdź do strony informacyjnej dla tego serwisu',
		'pt' => 'Ir para página de informação do serviço',
		'tr' => 'Servis için bilgi sayfasına git',
	),
	'table_caption' => array (
		'no' => 'Bruker innformasjon',
		'da' => 'Bruger information',
		'en' => 'User information',
		'de' => 'Benutzerdaten',
		'sv' => 'Användarinformation',
		'fi' => 'Käyttäjätiedot',
		'es' => 'Información del usuario',
		'nl' => 'Gerbuikersinformatie',
		'sl' => 'Podatki o uporabniku',
		'hu' => 'Felhasználói információk',
		'pt' => 'Informação do utilizador',
		'tr' => 'Kullanıcı bilgisi',
	),
	'table_summary' => array (
		'no' => 'Her listes den innformasjonen om deg som blir send til den tjenesten du er i ferd med å logge på',
		'da' => 'Informationer som vil blive sendt til den service du er ved at logge in på',
		'en' => 'List the information about you that is about to be transmitted to the service you are going to login to',
		'sv' => 'Visa den nformation om din användare som kommer att skickas till tjänsten som du är på väg att logga in i',
		'fi' => 'Näytä tietosi, joita ollaan siirtämässä palveluun',
		'es' => 'Muestra que información relativa a usted va a ser transmitida al servicio en el que se va a identificar',
		'nl' => 'De informatie die over jou bekend is en naar de service waarop je wilt inloggen verstuurd zal worden',
		'sl' => 'Seznam podatkov o vas, ki bodo posredovani storitvi, v katero se nameravate prijaviti',
		'hu' => 'Ezeket az adatokat fogjuk elküldeni Önről annak a szolgáltatásnak, ahová be kíván jelentkezni',
		'pt' => 'Listar a informação acerca de si que será enviada para o serviço no qual se está autenticar',
		'tr' => 'Girmek istediğiniz servise gönderilecek bilginizi listeleyin',
	),

);
